
#ifndef CDR_TYPES_HPP
#define CDR_TYPES_HPP

#include <boost/cstdint.hpp>

#include <boost/mpl/list.hpp>
#include <boost/mpl/contains.hpp>
#include <boost/mpl/insert_range.hpp>

namespace boost {
namespace archive {

namespace cdr {

// Bool has the same representation as in the ordered archive (8-bit)
typedef boost::mpl::list<bool> simple_types;

// Character types are very confusing in CDR...
typedef boost::mpl::list<char, wchar_t> char_types;

// CDR also allows 16-byte long doubles, but they don't have a corresponding 
// type in standard C++
typedef boost::mpl::list<float, double> real_types;

typedef boost::mpl::list<int8_t, uint8_t,
                         int16_t, uint16_t,
                         int32_t, uint32_t,
                         int64_t, uint64_t> integral_types;

// Strings and wstrings can also be serialized, but not directly

} // namespace cdr

// Define the types that can be serialized in a CDR archive
typedef boost::mpl::insert_range<
            cdr::integral_types, 
            boost::mpl::end<cdr::integral_types>::type,
            boost::mpl::insert_range<
                cdr::real_types,
                boost::mpl::end<cdr::real_types>::type,
                boost::mpl::insert_range<
                    cdr::char_types,
                    boost::mpl::end<cdr::char_types>::type,
                    cdr::simple_types
                >::type
            >::type
        >::type cdr_types;

} // namespace archive
} // namespace boost

#endif

