
#include <boost/cstdint.hpp>

#include "ordered_iarchive.hpp"
#include <boost/archive/basic_binary_iprimitive.ipp>

namespace boost {
namespace archive {

using marshall::Little;
using marshall::Big;

// Explicitly instantiate for this type of binary stream
template class basic_binary_iprimitive<ordered_iarchive<Big>, std::istream>;
template class ordered_iarchive_impl<ordered_iarchive<Big>, Big>;

template class basic_binary_iprimitive<ordered_iarchive<Little>, std::istream>;
template class ordered_iarchive_impl<ordered_iarchive<Little>, Little>;

// Instantiate save functions for these archive primitive types
typedef ordered_binary_iprimitive<ordered_iarchive<Big>, std::istream, Big> std_big_iprimitive;
typedef ordered_binary_iprimitive<ordered_iarchive<Little>, std::istream, Little> std_little_iprimitive;

using marshall::conversion_function16_t;
using marshall::conversion_function32_t;
using marshall::conversion_function64_t;

// Instantiate functions to convert and save specific data sizes
template<>
void ordered_load<1, Big>(std_big_iprimitive* primitive, void* address)
{
  primitive->load_binary(address, 1);
}
template<>
void ordered_load<1, Little>(std_little_iprimitive* primitive, void* address)
{
  primitive->load_binary(address, 1);
}

template<>
void ordered_load<2, Big>(std_big_iprimitive* primitive, void* address)
{
  do_ordered_load<uint16_t, conversion_function16_t, Big, std_big_iprimitive>(primitive, address);
}
template<>
void ordered_load<2, Little>(std_little_iprimitive* primitive, void* address)
{
  do_ordered_load<uint16_t, conversion_function16_t, Little, std_little_iprimitive>(primitive, address);
}

template<>
void ordered_load<4, Big>(std_big_iprimitive* primitive, void* address)
{
  do_ordered_load<uint32_t, conversion_function32_t, Big, std_big_iprimitive>(primitive, address);
}
template<>
void ordered_load<4, Little>(std_little_iprimitive* primitive, void* address)
{
  do_ordered_load<uint32_t, conversion_function32_t, Little, std_little_iprimitive>(primitive, address);
}

#ifndef BOOST_NO_INT64_T
template<>
void ordered_load<8, Big>(std_big_iprimitive* primitive, void* address)
{
  do_ordered_load<uint64_t, conversion_function64_t, Big, std_big_iprimitive>(primitive, address);
}
template<>
void ordered_load<8, Little>(std_little_iprimitive* primitive, void* address)
{
  do_ordered_load<uint64_t, conversion_function64_t, Little, std_little_iprimitive>(primitive, address);
}
#endif

} // namespace archive
} // namespace boost

