
#include <boost/cstdint.hpp>

#include "ordered_oarchive.hpp"
#include <boost/archive/basic_binary_oprimitive.ipp>

namespace boost {
namespace archive {

using marshall::Little;
using marshall::Big;

// Explicitly instantiate for this type of binary stream
template class basic_binary_oprimitive<ordered_oarchive<Big>, std::ostream>;
template class ordered_oarchive_impl<ordered_oarchive<Big>, Big>;

template class basic_binary_oprimitive<ordered_oarchive<Little>, std::ostream>;
template class ordered_oarchive_impl<ordered_oarchive<Little>, Little>;

// Instantiate save functions for these archive types
typedef ordered_binary_oprimitive<ordered_oarchive<Big>, std::ostream, Big> std_big_oprimitive;
typedef ordered_binary_oprimitive<ordered_oarchive<Little>, std::ostream, Little> std_little_oprimitive;

using marshall::conversion_function16_t;
using marshall::conversion_function32_t;
using marshall::conversion_function64_t;

// Instantiate functions to convert and save specific data sizes
template<>
void ordered_save<1, Big>(std_big_oprimitive* primitive, const void* address)
{
  primitive->save_binary(address, 1);
}
template<>
void ordered_save<1, Little>(std_little_oprimitive* primitive, const void* address)
{
  primitive->save_binary(address, 1);
}

template<>
void ordered_save<2, Big>(std_big_oprimitive* primitive, const void* address)
{
  do_ordered_save<uint16_t, conversion_function16_t, Big, std_big_oprimitive>(primitive, address);
}
template<>
void ordered_save<2, Little>(std_little_oprimitive* primitive, const void* address)
{
  do_ordered_save<uint16_t, conversion_function16_t, Little, std_little_oprimitive>(primitive, address);
}

template<>
void ordered_save<4, Big>(std_big_oprimitive* primitive, const void* address)
{
  do_ordered_save<uint32_t, conversion_function32_t, Big, std_big_oprimitive>(primitive, address);
}
template<>
void ordered_save<4, Little>(std_little_oprimitive* primitive, const void* address)
{
  do_ordered_save<uint32_t, conversion_function32_t, Little, std_little_oprimitive>(primitive, address);
}

#ifndef BOOST_NO_INT64_T
template<>
void ordered_save<8, Big>(std_big_oprimitive* primitive, const void* address)
{
  do_ordered_save<uint64_t, conversion_function64_t, Big, std_big_oprimitive>(primitive, address);
}
template<>
void ordered_save<8, Little>(std_little_oprimitive* primitive, const void* address)
{
  do_ordered_save<uint64_t, conversion_function64_t, Little, std_little_oprimitive>(primitive, address);
}
#endif

} // namespace archive
} // namespace boost

