
#ifndef XDR_MARSHALL_HPP
#define XDR_MARSHALL_HPP

#include <iostream>
#include <ostream>

#include <boost/serialization/access.hpp>
#include <boost/serialization/split_member.hpp>

#include "vector_marshall.hpp"

namespace xdr_marshall {

template<class T, size_t N>
class fixed_length : public T
{
public:
    fixed_length(void) : T() { resize(N); }
    fixed_length(const T & t) : T()
    {
      if (t.size() > N)
      {
        assign(t.begin(), t.begin() + N);
      }
      else
      {
        assign(t.begin(), t.end());

        resize(N);
      }
    }

private:
    friend class boost::serialization::access;

    template<class Archive>
    void save(Archive& ar, const unsigned version) const
    {
        if (size() <= N)
        {
            for (typename T::const_iterator it = begin(); it != end(); ++it)
                ar << (*it);

            unsigned count = N - size();
            while (count--)
                ar << typename T::value_type();
        }
        else
        {
            for (typename T::const_iterator it = begin(); it != (begin() + N); ++it)
                ar << (*it);
        }
    }

    template<class Archive>
    void load(Archive& ar, const unsigned version)
    {
        if (size() != N)
            resize(N);

        for (typename T::iterator it = begin(); it != end(); ++it)
            ar >> (*it);
    }

    template<class Archive>
    void serialize(Archive& ar, const unsigned version)
    {
        boost::serialization::split_member(ar, *this, version);
    }
};

/*
template<class T>
class variable_length : public T
{
public:
    variable_length(void) : T() {}
    variable_length(const T & t) : T(t) {}

private:
    friend class boost::serialization::access;

    template<class Archive>
    void save(Archive& ar, const unsigned version) const
    {
        uint32_t count = size();

        ar << count << static_cast<const T&>(*this);
    }

    template<class Archive>
    void load(Archive& ar, const unsigned version) 
    {
        uint32_t count = 0;
        ar >> count;
        
        resize(count);
        ar >> static_cast<T&>(*this);
    }

    template<class Archive>
    void serialize(Archive& ar, const unsigned version)
    {
        boost::serialization::split_member(ar, *this, version);
    }
};
*/

} // namespace xdr_marshall

template <class T, size_t N>
inline std::ostream& operator<< (std::ostream& os, const xdr_marshall::fixed_length<T, N>& value)
{
  return os << static_cast<const T&>(value);
}

/*
template <class T>
inline std::ostream& operator<< (std::ostream& os, const xdr_marshall::variable_length<T>& value)
{
  return os << static_cast<const T&>(value);
}
*/

#endif // XDR_MARSHALL_HPP

