
#ifndef XDR_TYPES_HPP
#define XDR_TYPES_HPP

#include <boost/cstdint.hpp>

#include <boost/mpl/list.hpp>
#include <boost/mpl/contains.hpp>
#include <boost/mpl/insert_range.hpp>

namespace boost {
namespace archive {

namespace xdr {

// Only 8-bit char types in strings
typedef boost::mpl::list<char, unsigned char> char_types;

// XDR allows 32- and 64-bit floating types
typedef boost::mpl::list<float, double> real_types;

// XDR allows only 32-bit and 64-bit integrals
typedef boost::mpl::list<int32_t, uint32_t,
                         int64_t, uint64_t> integral_types;

// Bools are permitted, but must be forced to 32-bit length

// Strings and wstrings can also be serialized, but not directly

} // namespace xdr

// Define the types that can be serialized in a CDR archive
typedef boost::mpl::insert_range<
            xdr::integral_types, 
            boost::mpl::end<xdr::integral_types>::type,
            boost::mpl::insert_range<
                xdr::real_types,
                boost::mpl::end<xdr::real_types>::type,
                xdr::char_types
            >::type
        >::type xdr_types;

} // namespace archive
} // namespace boost

#endif

